<?php
/**
 *  Please read the terms of the CLUF license attached to this module(cf "licences" folder)
 *
 * @author    Línea Gráfica E.C.E. S.L.
 * @copyright Lineagrafica.es - Línea Gráfica E.C.E. S.L. all rights reserved.
 * @license   https://www.lineagrafica.es/licenses/license_en.pdf
 *            https://www.lineagrafica.es/licenses/license_es.pdf
 *            https://www.lineagrafica.es/licenses/license_fr.pdf
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Filkers extends Module
{
    protected $config_form = false;
    protected $url = array(
        'production' => array(
            'APP' => 'https://app.filkers.com',
            'API' => 'https://apirest.filkers.com',
        ),
        'sandbox' => array(
            'APP' => 'https://integracion.filkers.ovxtest.com',
            'API' => 'https://integracion.filkers.ovxtest.com',
        ),
    );
    protected $endPoint = array(
        'REGISTER' => '/api/data/v1/ps/register',
        'CHANGE' => '/api/data/v1/ps/item/change',
    );
    protected $permissions = array(
        'categories' => array('GET' => 1, 'HEAD' => 1),
        'configurations' => array('GET' => 1, 'HEAD' => 1),
        'image_types' => array('GET' => 1, 'HEAD' => 1),
        'images' => array('GET' => 1, 'HEAD' => 1),
        'product_customization_fields' => array('GET' => 1, 'HEAD' => 1),
        'product_feature_values' => array('GET' => 1, 'HEAD' => 1),
        'product_features' => array('GET' => 1, 'HEAD' => 1),
        'product_option_values' => array('GET' => 1, 'HEAD' => 1),
        'product_options' => array('GET' => 1, 'HEAD' => 1),
        'product_suppliers' => array('GET' => 1, 'HEAD' => 1),
        'products' => array('GET' => 1, 'HEAD' => 1),
        'countries' => array('GET' => 1, 'HEAD' => 1),
        'currencies' => array('GET' => 1, 'HEAD' => 1),
        'languages' => array('GET' => 1, 'HEAD' => 1),
    );

    public function __construct()
    {
        $this->name = 'filkers';
        $this->tab = 'others';
        $this->version = '1.0.0';
        $this->author = 'Línea Gráfica';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Filkers.com');
        $this->description = $this->l('Connection module with Filkers.com service');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        include dirname(__FILE__) . '/sql/install.php';

        Configuration::updateValue('FILKERS_SANDBOX', '0');
        Configuration::updateValue('FILKERS_LIVEMODE', '0');
        Configuration::updateValue('FILKERS_AUTHORIZED', '0');
        Configuration::updateValue('FILKERS_APP_ID', '');
        Configuration::updateValue('FILKERS_APP_SECRET', '');

        return parent::install() &&
        $this->registerHook('backOfficeHeader') &&
        $this->registerHook('deleteProduct') &&
        $this->registerHook('actionProductDelete') &&
        $this->registerHook('addProduct') &&
        $this->registerHook('actionProductAdd') &&
        $this->registerHook('actionProductSave') &&
        $this->registerHook('updateProduct') &&
        $this->registerHook('actionProductUpdate') &&
        $this->registerHook('actionProductAdd') &&
        $this->registerHook('afterSaveProduct') &&
        $this->registerHook('actionUpdateQuantity') &&
        $this->registerHook('actionOrderStatusUpdate') &&
        $this->registerHook('actionOrderStatusPostUpdate') &&
        $this->registerHook('postUpdateOrderStatus') &&
        $this->registerHook('actionValidateOrder');
    }

    public function uninstall()
    {
        include dirname(__FILE__) . '/sql/uninstall.php';
        Configuration::deleteByName('FILKERS_SANDBOX');
        Configuration::deleteByName('FILKERS_LIVEMODE');
        Configuration::deleteByName('FILKERS_AUTHORIZED');
        Configuration::deleteByName('FILKERS_APP_ID');
        Configuration::deleteByName('FILKERS_APP_SECRET');
        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */		
        $output = '';
        if (((bool) Tools::isSubmit('submitFilkersModule')) == true) {
            $errors = $this->postProcess();
        }
        if (!empty($errors)) {
            foreach ($errors as $error) {
                $output .= $this->displayError($error);
            }
        }
        $this->context->smarty->assign('module_dir', $this->_path);
        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitFilkersModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
        . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        if (Tools::getValue('debug')) {
            $helper->currentIndex .= '&debug=true';
        }
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        if (Tools::getValue('debug')) {
            $form = array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Settings'),
                        'icon' => 'icon-cogs',
                    ),
                    'input' => array(
                        array(
                            'type' => 'text',
                            'name' => 'FILKERS_APP_ID',
                            'label' => $this->l('App ID'),
                            'col' => 4,
                        ),
                        array(
                            'type' => 'text',
                            'name' => 'FILKERS_APP_SECRET',
                            'label' => $this->l('App Secret'),
                            'col' => 4,
                        ),
                        array(
                            'type' => 'switch',
                            'label' => $this->l('Live Mode'),
                            'name' => 'FILKERS_LIVEMODE',
                            'is_bool' => true,
                            'desc' => $this->l('Use this mode instead the Cron Job, may affect product save time.'),
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => true,
                                    'label' => $this->l('Enabled'),
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => false,
                                    'label' => $this->l('Disabled'),
                                ),
                            ),
                        ),
                        array(
                            'type' => 'switch',
                            'label' => $this->l('SandBox mode'),
                            'name' => 'FILKERS_SANDBOX',
                            'is_bool' => true,
                            'desc' => $this->l('Use this module in SandBox mode. Only for development purpose.'),
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => true,
                                    'label' => $this->l('Enabled'),
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => false,
                                    'label' => $this->l('Disabled'),
                                ),
                            ),
                        ),
                        array(
                            'type' => 'connected',
                            'name' => 'FILKERS_AUTHORIZED',
                            'label' => $this->l('Authorization status'),
                            'col' => 3,
                            'value' => Configuration::get('FILKERS_AUTHORIZED', '0'),
                        ),
                    ),
                    'submit' => array(
                        'title' => $this->l('Save'),
                    ),
                ),
            );
        } else {
            $form = array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Settings'),
                        'icon' => 'icon-cogs',
                    ),
                    'input' => array(
                        array(
                            'type' => 'text',
                            'name' => 'FILKERS_APP_ID',
                            'label' => $this->l('App ID'),
                            'col' => 4,
                        ),
                        array(
                            'type' => 'text',
                            'name' => 'FILKERS_APP_SECRET',
                            'label' => $this->l('App Secret'),
                            'col' => 4,
                        ),
                        array(
                            'type' => 'connected',
                            'name' => 'FILKERS_AUTHORIZED',
                            'label' => $this->l('Authorization status'),
                            'col' => 3,
                            'value' => Configuration::get('FILKERS_AUTHORIZED', '0'),
                        ),
                    ),
                    'submit' => array(
                        'title' => $this->l('Save'),
                    ),
                ),
            );
        }
        $key = Configuration::get('FILKERS_APP_SECRET');
        if (Tools::getValue('debug') && !empty($key)) {
            $form['form']['input'][] = array(
                'col' => 6,
                'type' => 'copy',
                'name' => 'FILKERS_CRONJOB_URL',
                'label' => $this->l('Cron Job Url'),
                'value' => $this->getCronJobUrl(),
            );
        }
        return $form;
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        if (Tools::getValue('debug')) {
            return array(
                'FILKERS_SANDBOX' => Configuration::get('FILKERS_SANDBOX', '0'),
                'FILKERS_LIVEMODE' => Configuration::get('FILKERS_LIVEMODE', '0'),
                'FILKERS_APP_ID' => Configuration::get('FILKERS_APP_ID', ''),
                'FILKERS_APP_SECRET' => Configuration::get('FILKERS_APP_SECRET', ''),
            );
        } else {
            return array(
                'FILKERS_APP_ID' => Configuration::get('FILKERS_APP_ID', ''),
                'FILKERS_APP_SECRET' => Configuration::get('FILKERS_APP_SECRET', ''),
            );
        }        
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $errors = array();
        $form_values = $this->getConfigFormValues();        
        $appSecretOld = Configuration::get('FILKERS_APP_SECRET');

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }

        $appId = Configuration::get('FILKERS_APP_ID');
        $appSecret = Configuration::get('FILKERS_APP_SECRET');
        if (!empty($appId) && !empty($appSecret) && $appSecret != $appSecretOld) {
            $errors = array_merge($errors, $this->createWebservice());
        }
        if (empty($appId)) {
            $errors[] = $this->l('The App ID can\'t be empty.');
        }
        if (empty($appSecret)) {
            $errors[] = $this->l('The App Secret can\'t be empty.');
        }
        return $errors;
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name || Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
            $token = Tools::getAdminTokenLite('AdminFilkersAjax');
            if (version_compare(_PS_VERSION_, '1.6.1.0', '>=')) {
                Media::addJsDef(
                    array(
                        'filkers_token' => $token,
                        'filkers_connected' => $this->l('Connected'),
                        'filkers_disconnected' => $this->l('Not connected'),
                    )
                );
            } else {
                $this->context->smarty->assign(
                    array(
                        'filkers_token' => $token,
                        'filkers_connected' => $this->l('Connected'),
                        'filkers_disconnected' => $this->l('Not connected'),
                    )
                );

                return $this->context->smarty->fetch(
                    _PS_MODULE_DIR_ . $this->name
                    . DIRECTORY_SEPARATOR . 'views'
                    . DIRECTORY_SEPARATOR . 'templates'
                    . DIRECTORY_SEPARATOR . 'admin'
                    . DIRECTORY_SEPARATOR . 'javascript.tpl'
                );
            }
        }
    }

    public function enableWebservice($enable = 1)
    {
        Configuration::updateValue('PS_WEBSERVICE', $enable);
    }

    public function createWebservice()
    {
        $errors = array();
        $key = Configuration::get('FILKERS_APP_SECRET');
        if (!empty($key) && 32 >= Tools::strlen($key)) {
            if (!WebserviceKey::keyExists($key)) {
                $apiAccess = new WebserviceKey();
                $apiAccess->key = $key;
                $apiAccess->save();
                WebserviceKey::setPermissionForAccount($apiAccess->id, $this->permissions);
                $this->enableWebservice(1);
            }
            if (!$this->getAuthorization()) {
                $errors[] = $this->l('Can\'t get authorization with current data. Please revise it.');
            }
        } elseif (!empty($key) && Tools::strlen($key) > 32) {
            $errors[] = $this->l('The App Secret must be of 32 characters maximum.');
        }
        return $errors;
    }

    public function getShopUrl()
    {
        return Tools::getHttpHost(true) . __PS_BASE_URI__;
    }

    public function callApi($url = null, $json = null)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        return $http_code == 200 ? true : false;
    }

    public function getAuthorization()
    {
        $dataArr = array(
            'appId' => Configuration::get('FILKERS_APP_ID'),
            'appSecret' => Configuration::get('FILKERS_APP_SECRET'),
            'url' => $this->getShopUrl(),
        );
        $json = Tools::jsonEncode($dataArr);
        $url = (Configuration::get('FILKERS_SANDBOX') == 1
            ? $this->url['sandbox']['API']
            : $this->url['production']['API']
        ) . $this->endPoint['REGISTER'];
        if ($this->callApi($url, $json)) {
            Configuration::updateValue('FILKERS_AUTHORIZED', '1');
            return true;
        } else {
            Configuration::updateValue('FILKERS_AUTHORIZED', '0');
            return false;
        }
    }

    public function notifyChanges()
    {
        if ((int) Configuration::get('FILKERS_AUTHORIZED') != 1 ||
			!$this->hasPassedSeconds()
		) {
            return false;
        }
		Configuration::updateValue('FILKERS_LASTUPDATE', time());
		$success = false;
        $items = $this->getItems();
        if (!empty($items)) {
            $dataArr = array(
                'appId' => Configuration::get('FILKERS_APP_ID'),
                'appSecret' => Configuration::get('FILKERS_APP_SECRET'),
                'itemIds' => $items,
            );
            $json = Tools::jsonEncode($dataArr);
            $url = (Configuration::get('FILKERS_SANDBOX') == 1
                ? $this->url['sandbox']['API']
                : $this->url['production']['API']
            ) . $this->endPoint['CHANGE'];
            $ok = $this->callApi($url, $json);
            if ($ok) {
                $sql = 'UPDATE `' . _DB_PREFIX_ . 'filkers` SET `need_update` = 0, `date_sync` = NOW()';
                Db::getInstance()->execute($sql);
                $sql = 'DELETE FROM `' . _DB_PREFIX_ . 'filkers`
                    WHERE `id_product` NOT IN(SELECT DISTINCT `id_product` FROM `' . _DB_PREFIX_ . 'product`)';
                Db::getInstance()->execute($sql);
                $success = true;
            }
        }		
        return $success;
    }

    public function getItems()
    {
        $items = array();
        $sql = 'SELECT DISTINCT `id_product` FROM `' . _DB_PREFIX_ . 'filkers` WHERE `need_update` = 1';
        if ($rows = Db::getInstance()->executeS($sql)) {
            foreach ($rows as $row) {
                $items[] = $row['id_product'];
            }
        }
        return $items;
    }

    public function checkProducts($products = array())
    {
		if (!$this->hasPassedSeconds()) {
			return false;
		}
        $products = is_array($products) ? $products : array($products);
        if (!empty($products)) {
            $values = array();
            foreach ($products as $id_product) {
                $values[] = '(' . (int) $id_product . ',1,NOW())';
            }
            $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'filkers` (`id_product`,`need_update`,`date_add`)
                VALUES ' . implode(',', $values) . '
                ON DUPLICATE KEY UPDATE `need_update` = 1, `date_add` = NOW()';
            Db::getInstance()->execute($sql);
        }
    }

    public function hookActionUpdateQuantity($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookUpdateQuantity($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookactionProductAdd($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookActionProductUpdate($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookAfterSaveProduct($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookActionProductSave($params)
    {
        if ($this->context->controller instanceof AdminLegacyLayoutControllerCore ||
            $this->context->controller instanceof AdminProductsController ||
			$this->context->controller instanceof AdminController ||
            (
                $this->context->controller instanceof AdminModulesController &&
                pSQL(Tools::getValue('configure')) == $this->name
            )
        ) {
            if (!empty($params['id_product'])) {
                $this->checkProducts($params['id_product']);
            }
            if (Configuration::get('FILKERS_LIVEMODE') == 1) {
                $this->notifyChanges();
            }
        }
    }

    public function hookProductDelete($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookActionProductDelete($params)
    {
        return $this->hookActionProductSave($params);
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $this->hookActionValidateOrder($params);
    }

    public function hookPostUpdateOrderStatus($params)
    {
        $this->hookActionValidateOrder($params);
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $this->hookActionValidateOrder($params);
    }

    public function hookActionValidateOrder($params)
    {
        if (!empty($params['order']) && is_object($params['order'])) {
            $order = $params['order'];
            if (Validate::isLoadedObject($order)) {
                $products = array();
                $list = $order->getProductsDetail();
                if (!empty($list)) {
                    $products = array();
                    foreach ($list as $product) {
                        $products[] = $product['id_product'];
                    }
                    $this->checkProducts($products);
                    if (Configuration::get('FILKERS_LIVEMODE') == 1) {
                        $this->notifyChanges();
                    }
                }
            }
        }
    }

    public function getCronJobUrl($controller = 'cronjob')
    {
        $key = Configuration::get('FILKERS_APP_SECRET');
        return Context::getContext()->link->getModuleLink(
            $this->name,
            $controller
        ) . '?token=' . $key;
    }

    public function hasPassedSeconds($sec_passed = 5)
    {
		$time = Configuration::get('FILKERS_LASTUPDATE');
		$now = time();
		if (empty($time) || ($now - $time) > $sec_passed) {
			return true;
		}
		return false;
    }
	
	public function setLog()
	{
		$path = _PS_MODULE_DIR_ . $this->name . DIRECTORY_SEPARATOR . 'log.txt';
		$data = date('Y-m-d H:i:s') . ' - ' . get_class($this->context->controller) . "\n";
		$fp = fopen($path, 'a');
		fwrite($fp, $data);
	}
}
